/*
 * to		A simple program that can be used as the "transmitter"
 *		part of a TCP network stream connection.  It could
 *		be used for setting up inter-machine network pipes
 *		for programs like "tar"...
 *
 * Usage:	to [-b blocksize] [-p] [-t] [-v] host portno
 *
 * Version:	@(#)to		1.03	01/30/94
 *
 * Authors:	Fred N. van Kempen, <waltje@uwalt.nl.mugnet.org>
 *		Danny ter Haar, <danny@cistron.nl.mugnet.org>
 */
#include <sys/types.h>
#include <sys/socket.h>
#include <sys/time.h>
#include <arpa/inet.h>
#include <netinet/in.h>
#include <errno.h>
#include <netdb.h>
#include <string.h>
#include <stdlib.h>
#include <time.h>
#include <unistd.h>
#include <stdio.h>


#define BLOCK_SIZE	1024			/* default buffer size */


static char *Version = "@(#)to 1.03 (01/30/94";



static void
pr_sock(struct sockaddr_in *sip, int ports)
{
  if (ports == 1) {
	fprintf(stderr, "%u,%u,%u,%u,%u,%u\n",
		((sip->sin_addr.s_addr >> 0) & 255),
		((sip->sin_addr.s_addr >> 8) & 255),
		((sip->sin_addr.s_addr >> 16) & 255),
		((sip->sin_addr.s_addr >> 24) & 255),
		((sip->sin_port >> 0) & 255),
		((sip->sin_port >> 8) & 255));
  } else fprintf(stderr, "%s:%u\n",
		inet_ntoa(sip->sin_addr), ntohs(sip->sin_port));
}


static void
usage(void)
{
  fprintf(stderr, "Usage: to [-b blocksize] [-p] [-t] [-v] host port\n");
  exit(1);
}


int
main(int argc, char **argv)
{
  char hostname[128], *buff;
  struct sockaddr_in him;
  struct hostent *hp;
  struct tm *tm;
  struct timeval start, stop;
  time_t elap;
  int opt_p = 0;
  int opt_t = 0;
  int opt_v = 0;
  int b, i, j, s;
  unsigned long count;
  extern int getopt(), optind, opterr;

  opterr = 0;
  b = BLOCK_SIZE;
  while((i = getopt(argc, argv, "b:ptv")) != EOF) switch(i) {
	case 'b':
		b = atoi(optarg);
	 	if (b <= 0) usage();
		break;

	case 'p':
		opt_p = 1;
		break;

	case 't':
		opt_t = 1;
		break;

	case 'v':
		opt_v = 1;
		break;

	default:
		usage();
  }

  /* Two more arguments required. */
  if (optind != (argc - 2)) usage();

  /* Allocate a block buffer. */
  if ((buff = (char *) malloc(b)) == NULL) {
	fprintf(stderr, "to: cannot allocate buffer memory !\n");
	exit(1);
  }

  /* Fetch hostname and port number. */
  strncpy(hostname, argv[optind++], 128);
  i = atoi(argv[optind]);

  /* Convert hostname to an IP address. */
  if ((hp = gethostbyname(hostname)) == (struct hostent *)NULL) {
	herror(hostname);
	exit(1);
  }
  memcpy((char *) &him.sin_addr, hp->h_addr_list[0], hp->h_length);
  strcpy(hostname, hp->h_name);
  him.sin_family = AF_INET;
  him.sin_port = htons(i);
  if (opt_v ==1 ) {
	fprintf(stderr, "Sending to ");
	pr_sock(&him, opt_p);
  }

  /* Create a socket. */
  if ((s = socket(him.sin_family, SOCK_STREAM, 0)) < 0) {
	perror("socket");
	exit(1);
  }

  /* All done. Connect! */
  if (connect(s, (struct sockaddr *) &him, sizeof(him)) < 0) {
	perror("connect");
	(void) close(s);
	exit(1);
  }

  /* What time is it? */
  if (gettimeofday(&start, NULL) < 0) {
	fprintf(stderr, "to: cannot get time of day: %s !\n", strerror(errno));
	exit(1);
  }

  /* Start sending data until there is no more... */
  count = 0L;
  do {
	if ((i = read(0, buff, b)) <= 0) break;
	if (opt_v == 1) {
		printf("\rSent %08ld bytes", count);
		(void) fflush(stdout);
	}
	j = write(s, buff, i);
	if (j != i) {
		fprintf(stderr, "Huh?  Bad write %d of %d (%d)\n",
						j, i, errno);
		break;
	}
	count += (long) j;
  } while(1);

  /* What time is it? */
  if (gettimeofday(&stop, NULL) < 0) {
	fprintf(stderr, "to: cannot get time of day: %s !\n", strerror(errno));
	exit(1);
  }

  /* Check for any errors.. */
  if (i != 0) {
	fprintf(stderr, "Transfer aborted: %s\n", sys_errlist[errno]);
  }

  /* Close up! */
  (void) close(s);

  /* Number of milliseconds the program ran. */
  elap = ((stop.tv_sec - start.tv_sec) * 1000);
  elap += ((stop.tv_usec - start.tv_usec) / 1000);

  /* If desired by the caller, display the statistics. */
  if (opt_t == 1) {
	printf("%lu.%d seconds (%lu.%u Kbytes/sec)\n",
		(elap / 1000), (elap % 1000),
		(((count + 1023) / 1024) / (elap / 1000)),
		(((count + 1023) / 1024) % (elap / 1000)));
  }

  return(0);
}
